# Copyright (c) Microsoft Corporation.  All rights reserved.
# This file is auto-generated. Do not make manual changes.

from .protobuf_objects import ProseMicrosoftProgramSynthesisPythonDataInsightsInternals_pb2
from .protobuf_objects import ProseDispatchApisMicrosoftProgramSynthesisPythonDataInsightsInternals_pb2
from .protobuf_objects import ProseFetchApisMicrosoftProgramSynthesisPythonDataInsightsInternals_pb2
from .background_process import BackgroundProcess, _ProcessState
from .cache_manager import CacheManager
from .protobuf_objects.background_process_terminate import process_terminator
from .null_value_converter_utils import NullValueConverterUtils


class PatternFinderApi(object):
    def __init__(self, background_process, inputs, prepopulated_referenceId=None):
        """
        Construct a pattern finder from a sequence of input string .
        :param inputs: The set of strings to find patterns from.
        :return:
        """
        def _convert_to_pb_0(protobuf_object, python_object) -> None:
            """
            Converts the python_object and sets converted object in appropriate fields of protobuf_object. />.
            :param protobuf_object:
            :param python_object:
            :return:
            """
            if python_object is None:
                protobuf_object.IsNullValue = True
                return

            if len(python_object) == 0:
                protobuf_object.Value.extend(python_object)
                return

            for inner_py_object in python_object:
                inner_pb_object = protobuf_object.Value.add()
                NullValueConverterUtils.convert_to_pb_nullable_string(inner_pb_object, inner_py_object)

        assert background_process is not None
        self._background_process = background_process
        self._cache_manager = CacheManager(id(background_process))

        # if object was already created in C#, just initialize the referenceId.
        if prepopulated_referenceId is not None:
            self.referenceId = prepopulated_referenceId
            self._cache_manager.add_object(self.referenceId, self)
            return
        # else if object is constructed in Python, then create an equivalent in C#.

        # Create a request object for dispatcher
        message_request = ProseMicrosoftProgramSynthesisPythonDataInsightsInternals_pb2.PythonToCSharp()
        request = message_request.requestForDispatcher


        # Extract 'PatternFinder' object from request and set parameters
        pattern_finder_request = request.iPatternFinder
        _convert_to_pb_0(pattern_finder_request.i_In_PatternFinder___init__.inputs, inputs)
        message_response = ProseMicrosoftProgramSynthesisPythonDataInsightsInternals_pb2.CSharpToPython()

        # Communicate with Coreclr
        received_bytes = self._background_process.communicate(message_request)

        # If we didn't receive any bytes, raise Error
        if received_bytes is None:
            raise RuntimeError('Internal error has occurred. Please retry the entire scenario.')

        message_response.ParseFromString(received_bytes)
        response = message_response.responseFromDispatcher

        # Check if any exception was thrown by PatternFinder..ctor
        if response.WhichOneof('ScenarioOutput') == 'Exception':
            raise RuntimeError(response.Exception.message)
        # Ensure we have a valid referenceId
        if response.referenceId is 0:
            raise ValueError('Failed to create PatternFinder object.')
        self.referenceId = response.referenceId
        # Cache the object
        self._cache_manager.add_object(self.referenceId, self)

    def find(self):
        """
        Find the patterns in the input strings.
        :return:
        """
        def _convert_from_pb_0(protobuf_object):
            """
            Converts the protobuf_object into python_object equivalent. />.
            :param protobuf_object:
            :param python_object:
            :return:
            """
            if protobuf_object.IsNullValue:
                return None
            return [NullValueConverterUtils.convert_from_pb_nullable_string(python_object) for python_object in protobuf_object.Value]


        # Create a request object for dispatcher
        message_request = ProseMicrosoftProgramSynthesisPythonDataInsightsInternals_pb2.PythonToCSharp()
        request = message_request.requestForDispatcher

        request.referenceId = self.referenceId

        # Extract 'PatternFinder' object from request and set parameters
        pattern_finder_request = request.iPatternFinder
        pattern_finder_request.i_In_PatternFinder_Find.SetInParent()
        message_response = ProseMicrosoftProgramSynthesisPythonDataInsightsInternals_pb2.CSharpToPython()

        # Communicate with Coreclr
        received_bytes = self._background_process.communicate(message_request)

        # If we didn't receive any bytes, raise Error
        if received_bytes is None:
            raise RuntimeError('Internal error has occurred. Please retry the entire scenario.')

        message_response.ParseFromString(received_bytes)
        response = message_response.responseFromDispatcher

        # Check if any exception was thrown by PatternFinder.Find
        if response.WhichOneof('ScenarioOutput') == 'Exception':
            raise RuntimeError(response.Exception.message)

        # Process the result
        pattern_finder_response = response.iPatternFinder.i_Out_PatternFinder_Find
        to_return = _convert_from_pb_0(pattern_finder_response.result)
        return to_return

    def __del__(self):

        # If we haven't initialized successfully, just return
        if not hasattr(self, 'referenceId'):
            return
        # Create a request object for dispatcher
        message_request = ProseMicrosoftProgramSynthesisPythonDataInsightsInternals_pb2.PythonToCSharp()
        request = message_request.requestForDispatcher

        request.referenceId = self.referenceId
        request.DeleteFromCache.SetInParent()
        message_response = ProseMicrosoftProgramSynthesisPythonDataInsightsInternals_pb2.CSharpToPython()
        # Communicate with Coreclr
        received_bytes = self._background_process.communicate(message_request)

        # If we didn't receive any bytes, just return
        if received_bytes is None:
            return

        message_response.ParseFromString(received_bytes)
        response = message_response.responseFromDispatcher

        # Check if any exception was thrown by PatternFinder.__del__
        if response.WhichOneof('ScenarioOutput') == 'Exception':
            raise RuntimeError(response.Exception.message)

    __slots__ = ['_background_process', '_cache_manager', 'referenceId', '__weakref__']


class PatternFinder(PatternFinderApi):
    def __init__(self, inputs, prepopulated_referenceId=None):
        """
        Construct a pattern finder from a sequence of input string .
        :param inputs: The set of strings to find patterns from.
        :return:
        """
        background_process = BackgroundProcess(r'Microsoft.ProgramSynthesis.Python.DataInsights.Internals.dll', '03/06/2020 05:12:02', process_terminator)
        assert background_process._process_state == _ProcessState.RUNNING
        super().__init__(background_process, inputs, prepopulated_referenceId)

    def __del__(self):
        super().__del__()

    __slots__ = []

    @classmethod
    def prepopulated_object_builder(cls, prepopulated_referenceId):
        """
        Method to construct PatternFinder object if equivalent C# object is already created
        """

        return PatternFinder(None, prepopulated_referenceId)
