# Copyright (c) Microsoft Corporation.  All rights reserved.

from .profile import Profile, ProfileTypes, ProfilingStrategy, DataMode


class DefaultMode(DataMode):
    def __init__(self):
        super().__init__("AllValues", [])


class DefaultProfile(Profile):
    """A default profile that is used when no structure about the 
    given data values can be learned."""

    def __init__(self):
        super().__init__(ProfileTypes.NONE, ProfilingStrategy.ITERATIVE)
        self.supports_pyspark = True

    def covers(self, value):
        """Returns true
        
        :returns: Returns true.
        :rtype: bool
        """
        return True

    @property
    def modes(self):
        return [DefaultMode()]

    def with_values(self, new_values, processed_count):
        """Returns a default profile.
        
        :returns: Returns a default profile.
        :rtype: DefaultProfile
        """
        return DefaultProfile()

    @property
    def active(self):
        """Returns true
        
        :returns: Returns true.
        :rtype: bool
        """
        return True

    def _with_values_pyspark(self, df, column):
        return DefaultProfile()

    def signature(self, value):
        """Returns the default signature (True, 0)
        
        :returns: Returns a constant signature.
        :rtype: Tuple[bool, int]
        """
        return (True, 0)
